@@ -12,9 +12,11 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "soup-connection.h"
 #include "soup-message.h"
 #include "soup-message-private.h"
 #include "soup-socket.h"
+#include "soup-ssl.h"
 
 typedef enum {
 	SOUP_MESSAGE_IO_CLIENT,
@@ -35,6 +37,7 @@ typedef enum {
 
 typedef struct {
 	SoupSocket           *sock;
+	SoupConnection       *conn;
 	SoupMessageIOMode     mode;
 
 	SoupMessageIOState    read_state;
@@ -63,23 +66,28 @@ typedef struct {
  */
 #define dummy_to_make_emacs_happy {
 #define SOUP_MESSAGE_IO_PREPARE_FOR_CALLBACK { gboolean cancelled; g_object_ref (msg);
-#define SOUP_MESSAGE_IO_RETURN_IF_CANCELLED_OR_PAUSED cancelled = (msg->priv->io_data != io); g_object_unref (msg); if (cancelled || !io->read_tag || !io->write_tag) return; }
-#define SOUP_MESSAGE_IO_RETURN_VAL_IF_CANCELLED_OR_PAUSED(val) cancelled = (msg->priv->io_data != io); g_object_unref (msg); if (cancelled || !io->read_tag || !io->write_tag) return val; }
+#define SOUP_MESSAGE_IO_RETURN_IF_CANCELLED_OR_PAUSED cancelled = (priv->io_data != io); g_object_unref (msg); if (cancelled || !io->read_tag || !io->write_tag) return; }
+#define SOUP_MESSAGE_IO_RETURN_VAL_IF_CANCELLED_OR_PAUSED(val) cancelled = (priv->io_data != io); g_object_unref (msg); if (cancelled || !io->read_tag || !io->write_tag) return val; }
 
 #define RESPONSE_BLOCK_SIZE 8192
 
 static void
 io_cleanup (SoupMessage *msg)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
-
-	if (!io)
-		return;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io;
 
 	soup_message_io_stop (msg);
 
+	io = priv->io_data;
+	if (!io)
+		return;
+	priv->io_data = NULL;
+
 	if (io->sock)
 		g_object_unref (io->sock);
+	if (io->conn)
+		g_object_unref (io->conn);
 
 	if (io->read_buf)
 		g_byte_array_free (io->read_buf, TRUE);
@@ -88,7 +96,6 @@ io_cleanup (SoupMessage *msg)
 	g_string_free (io->write_buf, TRUE);
 
 	g_free (io);
-	msg->priv->io_data = NULL;
 }
 
 /**
@@ -101,7 +108,8 @@ io_cleanup (SoupMessage *msg)
 void
 soup_message_io_stop (SoupMessage *msg)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 
 	if (!io)
 		return;
@@ -121,6 +129,12 @@ soup_message_io_stop (SoupMessage *msg)
 
 	if (io->read_state != SOUP_MESSAGE_IO_STATE_DONE)
 		soup_socket_disconnect (io->sock);
+	else if (io->conn) {
+		SoupConnection *conn = io->conn;
+		io->conn = NULL;
+		soup_connection_release (conn);
+		g_object_unref (conn);
+	}
 }
 
 #define SOUP_MESSAGE_IO_EOL            "\r\n"
@@ -144,7 +158,8 @@ soup_message_io_finished (SoupMessage *msg)
 static void
 io_error (SoupSocket *sock, SoupMessage *msg)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 
 	/* Closing the connection to signify EOF is sometimes ok */
 	if (io->read_state == SOUP_MESSAGE_IO_STATE_BODY &&
@@ -154,15 +169,26 @@ io_error (SoupSocket *sock, SoupMessage *msg)
 		return;
 	}
 
-	if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code))
-		soup_message_set_status (msg, SOUP_STATUS_IO_ERROR);
+	if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code)) {
+		GError *err = g_object_get_data (G_OBJECT (sock),
+						 "SoupSocket-last_error");
+
+		if (err && err->domain == SOUP_SSL_ERROR) {
+			soup_message_set_status_full (msg,
+						      SOUP_STATUS_SSL_FAILED,
+						      err->message);
+		} else
+			soup_message_set_status (msg, SOUP_STATUS_IO_ERROR);
+	}
+
 	soup_message_io_finished (msg);
 }
 
 static gboolean
 read_metadata (SoupMessage *msg, const char *boundary)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 	SoupSocketIOStatus status;
 	char read_buf[RESPONSE_BLOCK_SIZE];
 	guint boundary_len = strlen (boundary);
@@ -195,7 +221,8 @@ read_metadata (SoupMessage *msg, const char *boundary)
 static gboolean
 read_body_chunk (SoupMessage *msg)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 	SoupSocketIOStatus status;
 	char read_buf[RESPONSE_BLOCK_SIZE];
 	guint len = sizeof (read_buf);
@@ -248,7 +275,8 @@ read_body_chunk (SoupMessage *msg)
 static gboolean
 write_data (SoupMessage *msg, const char *data, guint len)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 	SoupSocketIOStatus status;
 	gsize nwrote;
 
@@ -315,7 +343,8 @@ static void io_read (SoupSocket *sock, SoupMessage *msg);
 static void
 io_write (SoupSocket *sock, SoupMessage *msg)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 
  write_more:
 	switch (io->write_state) {
@@ -355,7 +384,7 @@ io_write (SoupSocket *sock, SoupMessage *msg)
 				 */
 			}
 		} else if (io->mode == SOUP_MESSAGE_IO_CLIENT &&
-			   msg->priv->msg_flags & SOUP_MESSAGE_EXPECT_CONTINUE) {
+			   priv->msg_flags & SOUP_MESSAGE_EXPECT_CONTINUE) {
 			/* Need to wait for the Continue response */
 			io->write_state = SOUP_MESSAGE_IO_STATE_BLOCKING;
 			io->read_state = SOUP_MESSAGE_IO_STATE_HEADERS;
@@ -488,7 +517,8 @@ io_write (SoupSocket *sock, SoupMessage *msg)
 static void
 io_read (SoupSocket *sock, SoupMessage *msg)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 	guint status;
 
  read_more:
@@ -539,7 +569,7 @@ io_read (SoupSocket *sock, SoupMessage *msg)
 				io->read_state = SOUP_MESSAGE_IO_STATE_HEADERS;
 			}
 		} else if (io->mode == SOUP_MESSAGE_IO_SERVER &&
-			   (msg->priv->msg_flags & SOUP_MESSAGE_EXPECT_CONTINUE)) {
+			   (priv->msg_flags & SOUP_MESSAGE_EXPECT_CONTINUE)) {
 			/* The client requested a Continue response. */
 			io->write_state = SOUP_MESSAGE_IO_STATE_HEADERS;
 			io->read_state = SOUP_MESSAGE_IO_STATE_BLOCKING;
@@ -650,6 +680,7 @@ new_iostate (SoupMessage *msg, SoupSocket *sock, SoupMessageIOMode mode,
 	     SoupMessageParseHeadersFn parse_headers_cb,
 	     gpointer user_data)
 {
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
 	SoupMessageIOData *io;
 
 	io = g_new0 (SoupMessageIOData, 1);
@@ -663,7 +694,7 @@ new_iostate (SoupMessage *msg, SoupSocket *sock, SoupMessageIOMode mode,
 	io->write_encoding   = SOUP_TRANSFER_UNKNOWN;
 
 	io->read_meta_buf    = g_byte_array_new ();
-	if (!(msg->priv->msg_flags & SOUP_MESSAGE_OVERWRITE_CHUNKS))
+	if (!(priv->msg_flags & SOUP_MESSAGE_OVERWRITE_CHUNKS))
 		io->read_buf = g_byte_array_new ();
 	io->write_buf        = g_string_new (NULL);
 
@@ -677,9 +708,9 @@ new_iostate (SoupMessage *msg, SoupSocket *sock, SoupMessageIOMode mode,
 	io->read_state  = SOUP_MESSAGE_IO_STATE_NOT_STARTED;
 	io->write_state = SOUP_MESSAGE_IO_STATE_NOT_STARTED;
 
-	if (msg->priv->io_data)
+	if (priv->io_data)
 		io_cleanup (msg);
-	msg->priv->io_data = io;
+	priv->io_data = io;
 	return io;
 }
 
@@ -687,6 +718,7 @@ new_iostate (SoupMessage *msg, SoupSocket *sock, SoupMessageIOMode mode,
  * soup_message_io_client:
  * @msg: a #SoupMessage
  * @sock: socket to send @msg across
+ * @conn: the connection that owns @sock (or %NULL)
  * @get_headers_cb: callback function to generate request headers
  * @parse_headers_cb: callback function to parse response headers
  * @user_data: data to pass to the callbacks
@@ -697,6 +729,7 @@ new_iostate (SoupMessage *msg, SoupSocket *sock, SoupMessageIOMode mode,
  **/
 void
 soup_message_io_client (SoupMessage *msg, SoupSocket *sock,
+			SoupConnection *conn,
 			SoupMessageGetHeadersFn get_headers_cb,
 			SoupMessageParseHeadersFn parse_headers_cb,
 			gpointer user_data)
@@ -706,6 +739,9 @@ soup_message_io_client (SoupMessage *msg, SoupSocket *sock,
 	io = new_iostate (msg, sock, SOUP_MESSAGE_IO_CLIENT,
 			  get_headers_cb, parse_headers_cb, user_data);
 
+	if (conn)
+		io->conn = g_object_ref (conn);
+
 	io->read_body       = &msg->response;
 	io->write_body      = &msg->request;
 
@@ -752,7 +788,8 @@ soup_message_io_server (SoupMessage *msg, SoupSocket *sock,
 void  
 soup_message_io_pause (SoupMessage *msg)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 
 	g_return_if_fail (io != NULL);
 
@@ -775,7 +812,8 @@ soup_message_io_pause (SoupMessage *msg)
 void  
 soup_message_io_unpause (SoupMessage *msg)
 {
-	SoupMessageIOData *io = msg->priv->io_data;
+	SoupMessagePrivate *priv = SOUP_MESSAGE_GET_PRIVATE (msg);
+	SoupMessageIOData *io = priv->io_data;
 
 	g_return_if_fail (io != NULL);
 
