@@ -300,6 +300,16 @@ soup_server_get_protocol (SoupServer *server)
 		return SOUP_PROTOCOL_HTTP;
 }
 
+SoupSocket *
+soup_server_get_listener (SoupServer *server)
+{
+	SoupServerPrivate *priv;
+
+	g_return_val_if_fail (SOUP_IS_SERVER (server), NULL);
+	priv = SOUP_SERVER_GET_PRIVATE (server);
+
+	return priv->listen_sock;
+}
 
 static void start_request (SoupServer *, SoupSocket *);
 
