@@ -763,13 +763,16 @@ soup_socket_client_new_async (const char *hostname, guint port,
 			      SoupSocketCallback callback, gpointer user_data)
 {
 	SoupSocket *sock;
+	SoupAddress *addr;
 
 	g_return_val_if_fail (hostname != NULL, NULL);
 
 	sock = g_object_new (SOUP_TYPE_SOCKET,
 			     SOUP_SOCKET_SSL_CREDENTIALS, ssl_creds,
 			     NULL);
-	soup_socket_connect (sock, soup_address_new (hostname, port));
+	addr = soup_address_new (hostname, port);
+	soup_socket_connect (sock, addr);
+	g_object_unref (addr);
 
 	if (callback) {
 		soup_signal_connect_once (sock, "connect_result",
@@ -796,6 +799,7 @@ soup_socket_client_new_sync (const char *hostname, guint port,
 {
 	SoupSocket *sock;
 	SoupSocketPrivate *priv;
+	SoupAddress *addr;
 	guint status;
 
 	g_return_val_if_fail (hostname != NULL, NULL);
@@ -805,7 +809,9 @@ soup_socket_client_new_sync (const char *hostname, guint port,
 			     NULL);
 	priv = SOUP_SOCKET_GET_PRIVATE (sock);
 	priv->non_blocking = FALSE;
-	status = soup_socket_connect (sock, soup_address_new (hostname, port));
+	addr = soup_address_new (hostname, port);
+	status = soup_socket_connect (sock, addr);
+	g_object_unref (addr);
 
 	if (!SOUP_STATUS_IS_SUCCESSFUL (status)) {
 		g_object_unref (sock);
@@ -1002,7 +1008,11 @@ socket_read_watch (GIOChannel *chan, GIOCondition cond, gpointer user_data)
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
 	priv->read_tag = 0;
-	g_signal_emit (sock, signals[READABLE], 0);
+
+	if (cond & (G_IO_ERR | G_IO_HUP))
+		soup_socket_disconnect (sock);
+	else
+		g_signal_emit (sock, signals[READABLE], 0);
 
 	return FALSE;
 }
@@ -1041,7 +1051,8 @@ read_from_network (SoupSocket *sock, gpointer buffer, gsize len, gsize *nread)
 
 		if (!priv->read_tag) {
 			priv->read_tag =
-				g_io_add_watch (priv->iochannel, cond,
+				g_io_add_watch (priv->iochannel,
+						cond | G_IO_HUP | G_IO_ERR,
 						socket_read_watch, sock);
 		}
 		return SOUP_SOCKET_WOULD_BLOCK;
@@ -1194,13 +1205,17 @@ soup_socket_read_until (SoupSocket *sock, gpointer buffer, gsize len,
 }
 
 static gboolean
-socket_write_watch (GIOChannel *chan, GIOCondition condition, gpointer user_data)
+socket_write_watch (GIOChannel *chan, GIOCondition cond, gpointer user_data)
 {
 	SoupSocket *sock = user_data;
 	SoupSocketPrivate *priv = SOUP_SOCKET_GET_PRIVATE (sock);
 
 	priv->write_tag = 0;
-	g_signal_emit (sock, signals[WRITABLE], 0);
+
+	if (cond & (G_IO_ERR | G_IO_HUP))
+		soup_socket_disconnect (sock);
+	else
+		g_signal_emit (sock, signals[WRITABLE], 0);
 
 	return FALSE;
 }
@@ -1284,7 +1299,8 @@ soup_socket_write (SoupSocket *sock, gconstpointer buffer,
 	}
 
 	priv->write_tag =
-		g_io_add_watch (priv->iochannel, cond, 
+		g_io_add_watch (priv->iochannel,
+				cond | G_IO_HUP | G_IO_ERR, 
 				socket_write_watch, sock);
 	g_mutex_unlock (priv->iolock);
 	return SOUP_SOCKET_WOULD_BLOCK;
