@@ -11,6 +11,8 @@
 
 #include "soup-uri.h"
 
+static void append_uri_encoded (GString *str, const char *in, const char *extra_enc_chars);
+
 static inline SoupProtocol
 soup_uri_get_protocol (const char *proto, int len)
 {
@@ -40,8 +42,6 @@ soup_protocol_default_port (SoupProtocol proto)
 		return 0;
 }
 
-static void append_uri_encoded (GString *str, const char *in, const char *extra_enc_chars);
-
 /**
  * soup_uri_new_with_base:
  * @base: a base URI
@@ -255,11 +255,24 @@ soup_uri_new (const char *uri_string)
 	return uri;
 }
 
+
+static inline void
+append_uri (GString *str, const char *in, const char *extra_enc_chars,
+	    gboolean pre_encoded)
+{
+	if (pre_encoded)
+		g_string_append (str, in);
+	else
+		append_uri_encoded (str, in, extra_enc_chars);
+}
+
 /**
  * soup_uri_to_string:
  * @uri: a #SoupUri
  * @just_path: if %TRUE, output just the path and query portions
  *
+ * Returns a string representing @uri.
+ *
  * Return value: a string representing @uri, which the caller must free.
  **/
 char *
@@ -267,6 +280,7 @@ soup_uri_to_string (const SoupUri *uri, gboolean just_path)
 {
 	GString *str;
 	char *return_result;
+	gboolean pre_encoded = uri->broken_encoding;
 
 	/* IF YOU CHANGE ANYTHING IN THIS FUNCTION, RUN
 	 * tests/uri-parsing AFTERWARD.
@@ -279,10 +293,10 @@ soup_uri_to_string (const SoupUri *uri, gboolean just_path)
 	if (uri->host && !just_path) {
 		g_string_append (str, "//");
 		if (uri->user) {
-			append_uri_encoded (str, uri->user, ":;@/");
+			append_uri (str, uri->user, ":;@/", pre_encoded);
 			g_string_append_c (str, '@');
 		}
-		append_uri_encoded (str, uri->host, ":/");
+		append_uri (str, uri->host, ":/", pre_encoded);
 		if (uri->port && uri->port != soup_protocol_default_port (uri->protocol))
 			g_string_append_printf (str, ":%d", uri->port);
 		if (!uri->path && (uri->query || uri->fragment))
@@ -290,17 +304,17 @@ soup_uri_to_string (const SoupUri *uri, gboolean just_path)
 	}
 
 	if (uri->path && *uri->path)
-		append_uri_encoded (str, uri->path, "?");
+		append_uri (str, uri->path, "?", pre_encoded);
 	else if (just_path)
 		g_string_append_c (str, '/');
 
 	if (uri->query) {
 		g_string_append_c (str, '?');
-		append_uri_encoded (str, uri->query, NULL);
+		append_uri (str, uri->query, NULL, pre_encoded);
 	}
 	if (uri->fragment && !just_path) {
 		g_string_append_c (str, '#');
-		append_uri_encoded (str, uri->fragment, NULL);
+		append_uri (str, uri->fragment, NULL, pre_encoded);
 	}
 
 	return_result = str->str;
@@ -309,6 +323,14 @@ soup_uri_to_string (const SoupUri *uri, gboolean just_path)
 	return return_result;
 }
 
+/**
+ * soup_uri_copy:
+ * @uri: a #SoupUri
+ *
+ * Copies @uri
+ *
+ * Return value: a copy of @uri, which must be freed with soup_uri_free()
+ **/
 SoupUri *
 soup_uri_copy (const SoupUri *uri)
 {
@@ -326,9 +348,21 @@ soup_uri_copy (const SoupUri *uri)
 	dup->query    = g_strdup (uri->query);
 	dup->fragment = g_strdup (uri->fragment);
 
+	dup->broken_encoding = uri->broken_encoding;
+
 	return dup;
 }
 
+/**
+ * soup_uri_copy_root:
+ * @uri: a #SoupUri
+ *
+ * Copies the protocol, host, and port of @uri into a new #SoupUri
+ * (all other fields in the new URI will be empty.)
+ *
+ * Return value: a partial copy of @uri, which must be freed with
+ * soup_uri_free()
+ **/
 SoupUri *
 soup_uri_copy_root (const SoupUri *uri)
 {
@@ -354,22 +388,37 @@ parts_equal (const char *one, const char *two)
 	return !strcmp (one, two);
 }
 
+/**
+ * soup_uri_equal:
+ * @uri1: a #SoupUri
+ * @uri2: another #SoupUri
+ *
+ * Tests whether or not @uri1 and @uri2 are equal in all parts
+ *
+ * Return value: %TRUE or %FALSE
+ **/
 gboolean 
-soup_uri_equal (const SoupUri *u1, const SoupUri *u2)
+soup_uri_equal (const SoupUri *uri1, const SoupUri *uri2)
 {
-	if (u1->protocol != u2->protocol              ||
-	    u1->port     != u2->port                  ||
-	    !parts_equal (u1->user, u2->user)         ||
-	    !parts_equal (u1->passwd, u2->passwd)     ||
-	    !parts_equal (u1->host, u2->host)         ||
-	    !parts_equal (u1->path, u2->path)         ||
-	    !parts_equal (u1->query, u2->query)       ||
-	    !parts_equal (u1->fragment, u2->fragment))
+	if (uri1->protocol != uri2->protocol              ||
+	    uri1->port     != uri2->port                  ||
+	    !parts_equal (uri1->user, uri2->user)         ||
+	    !parts_equal (uri1->passwd, uri2->passwd)     ||
+	    !parts_equal (uri1->host, uri2->host)         ||
+	    !parts_equal (uri1->path, uri2->path)         ||
+	    !parts_equal (uri1->query, uri2->query)       ||
+	    !parts_equal (uri1->fragment, uri2->fragment))
 		return FALSE;
 
 	return TRUE;
 }
 
+/**
+ * soup_uri_free:
+ * @uri: a #SoupUri
+ *
+ * Frees @uri.
+ **/
 void
 soup_uri_free (SoupUri *uri)
 {
@@ -469,6 +518,15 @@ soup_uri_decode (char *part)
 	} while (*s++);
 }
 
+/**
+ * soup_uri_uses_default_port:
+ * @uri: a #SoupUri
+ *
+ * Tests if @uri uses the default port for its protocol. (Eg, 80 for
+ * http.)
+ *
+ * Return value: %TRUE or %FALSE
+ **/
 gboolean
 soup_uri_uses_default_port (const SoupUri *uri)
 {
